var ProductGroupedApp = {};

ProductGroupedApp.currencyParts = function(value){
  function padNumber(num, size){
      var s = num+"";
      while (s.length < size) s = "0" + s;
      return s;
  }

  var currencyFraction = value % 100;
  var currencyWhole = (value - currencyFraction) / 100;
  return [currencyWhole, padNumber(currencyFraction, 2)];
};

ProductGroupedApp.currencyPartsAsString = function(value){
  var parts = ProductGroupedApp.currencyParts(value);
  return '$' + parts[0] + '.' + parts[1];
};

// tabs
ProductGroupedApp.DeliveryOptionTab = React.createClass({
  handleSelected: function(){
    this.props.onSelected(this.props.option);
  },
  render: function(){
    return <li className={ classNames({'selected':this.props.isSelected, 'border-radius':true}) }
               onClick={ this.handleSelected }>
                         { this.props.option.text }
           </li>;
  }
});

ProductGroupedApp.DeliveryOptionTabs = React.createClass({
  render: function(){
    var tabs = this.props.options.map(function(option){
      return <ProductGroupedApp.DeliveryOptionTab key           = { option.id }
                                                  option        = { option }
                                                  isSelected    = { option.id == this.props.selectedId }
                                                  onSelected    = { this.props.onTabSelected }
                                                  deliveryDays  = { option.deliveryDays } />;
    }, this);

    return <ul className="delivery-option-tabs clearfix" role="tablist">
             { tabs }
           </ul>;
  }
});


// filtering
ProductGroupedApp.FilterImageCriteron = React.createClass({
  handleSelected: function(){
    if(!this.props.disabled)
      this.props.onSelected(this.props.criteron.title);
  },
  render: function(){

    return (
      <li className={ classNames({'selected':this.props.isSelected,'disabled':this.props.disabled}) }
          onClick={ this.handleSelected }>
        <h5>{ this.props.criteron.title }</h5>
        <div className="selector">
          <img src={ this.props.criteron.image } className="img-responsive center-block" />
          <div className="disabled-overlay" />
        </div>
        <p>{ this.props.criteron.price_string }</p>
      </li>
    );
  }
});

ProductGroupedApp.FilterColorCriteron = React.createClass({
  handleSelected: function(){
    if(!this.props.disabled)
      this.props.onSelected(this.props.criteron.title);
  },
  render: function(){

    return (
      <li className={ classNames({'selected':this.props.isSelected,'disabled':this.props.disabled}) }
          onClick={ this.handleSelected }>
        <h5>{ this.props.criteron.title }</h5>
        <div className="selector">
          <div style={{background: this.props.criteron.title.toLowerCase() }}/>
          <div className="disabled-overlay" />
        </div>
        <p>{ this.props.criteron.price_string }</p>
      </li>
    );
  }
});

ProductGroupedApp.FilterSquare = React.createClass({
  handleCriteronSelected: function(filter_criteron){
    this.props.onCriteronSelected(this.props.level.title, filter_criteron);
  },
  render: function(){

    var criteria = []
    // include only those criteria with products
    this.props.level.options.forEach(function(criteron){

      if(this.props.products.some(function(product){
          return product.filters[this.props.level.title] === criteron.title;
          }, this)){

          var enabled = this.props.products_for_selected_delivery_option.some(function(product){
            return product.filters[this.props.level.title] === criteron.title;
          }, this);

          criteria.push(React.createElement(this.props.selector, {
                                                                  key: criteron.title,
                                                                  criteron:   criteron,
                                                                  isSelected: criteron.title === this.props.selected,
                                                                  onSelected: this.handleCriteronSelected,
                                                                  disabled: !enabled
                                                                 }));
      }
    }, this);



    return (
      <div>
        <h4>
          <strong>{ this.props.level.title }:</strong>
          <ProductGroupedApp.FilterComparison fields={ this.props.level.comparison_fields } options={ this.props.level.options } />
          <span className="filter-value">{ this.props.selected }</span>
        </h4>
        <ul className="list-inline">
          { criteria }
        </ul>
      </div>
    );
  }
});

ProductGroupedApp.FilterDropDown = React.createClass({
  handleCriteronSelected: function(value){
    this.props.onCriteronSelected(this.props.level.title, value);
  },
  render: function(){

    var options = [];
    // include only those criteria with products
    this.props.level.options.sort(function(a,b){
      return a.price - b.price;
    }).forEach(function(criteron){

      if(this.props.products.some(function(product){
          return product.filters[this.props.level.title] === criteron.title;
          }, this)){

          var label = capitalizeString(criteron.title);
          if(criteron.price_string){
            label = label + ' (' + criteron.price_string + ')';
          }

          var enabled = this.props.products_for_selected_delivery_option.some(function(product){
            return product.filters[this.props.level.title] === criteron.title;
          }, this);

          options.push({
                        value: criteron.title,
                        label: label,
                        disabled: !enabled
                      });
     }
    }, this);

    return (
      <div className="row">
        <div className="col-xs-3">
          <h4>
            <strong>{ this.props.level.title }:</strong>
            <ProductGroupedApp.FilterComparison fields={ this.props.level.comparison_fields } options={ this.props.level.options } />
          </h4>
        </div>
        <div className="col-xs-4 col-md-5">
          <Select options={ options }
                  value={ ''+this.props.selected }
                  clearable={ false }
                  searchable={ false }
                  onChange={ this.handleCriteronSelected } />
        </div>
      </div>
    );

    function capitalizeString(str){
      return str.split(' ').map(function(word){
        return word.charAt(0).toUpperCase() + word.slice(1);
      }).join(' ');
    }
  }
});

ProductGroupedApp.Filters = React.createClass({
  render: function(){

    var levels = [];
    var previous_levels = [];
    var lowest_price_for_selection;

    this.props.levels.forEach(function(level, i){

      var products = filteredProducts.call(this);
      var filter;

      // do the products include this option?
      if(products.some(function(product){
          return product.filters[level.title] && level.options.map(function(option){return option.title}).indexOf(product.filters[level.title]) > -1;
        })){

        // top level show unit price
        // iterate through the filter level's options
        level.options.forEach(function(option){
          // we're looking for the price of the cheapest product with matching attribute
          var lowest_matching_product_price = products.reduce(function(memo, product){
            if(product.filters[level.title] === option.title && product.price < memo){
              memo = product.price;
            }
            return memo;
          }, Number.POSITIVE_INFINITY);

          option.price = lowest_matching_product_price;

          if(i===0 && this.props.selected[level.title] === option.title){
            lowest_price_for_selection = lowest_matching_product_price;
          }

          option.price_string = null;
        }, this);

        if(i===0){
          level.options.forEach(function(option){
            option.price_string = ProductGroupedApp.currencyPartsAsString(option.price);
          });
        }

        if(i>0 && lowest_price_for_selection){
          level.options.forEach(function(option){
            if(option.price !== Number.POSITIVE_INFINITY && (option.price - lowest_price_for_selection > 0)){
              option.price_string = '+' + ProductGroupedApp.currencyPartsAsString(option.price - lowest_price_for_selection);
            }
          });
        }

        var products_for_selected_delivery_option = products.filter(function(product){
           return product.delivery_options.filter(function(product_delivery_option){
             return this.props.selected_delivery_options.indexOf(product_delivery_option) > -1;
           }, this).length;
        }, this);

        if(level.title.toLowerCase() === 'color'){

          filter = <ProductGroupedApp.FilterSquare level              = { level }
                                                   selector           = { ProductGroupedApp.FilterColorCriteron }
                                                   selected           = { this.props.selected[level.title] }
                                                   products           = { products }
                                                   products_for_selected_delivery_option = { products_for_selected_delivery_option }
                                                   onCriteronSelected = { this.props.onCriteronSelected } />;

        } else if(level.options[0] && level.options[0].image){

          filter = <ProductGroupedApp.FilterSquare level              = { level }
                                                   selector           = { ProductGroupedApp.FilterImageCriteron }
                                                   selected           = { this.props.selected[level.title] }
                                                   products           = { products }
                                                   products_for_selected_delivery_option = { products_for_selected_delivery_option }
                                                   onCriteronSelected = { this.props.onCriteronSelected } />;
        } else {

          filter = <ProductGroupedApp.FilterDropDown level              = { level }
                                                     selected           = { this.props.selected[level.title] }
                                                     products           = { products }
                                                     products_for_selected_delivery_option = { products_for_selected_delivery_option }
                                                     onCriteronSelected = { this.props.onCriteronSelected } />;
        }

      }

      if(filter){
        levels.push(<div className="filter"
                         key={ level.title }>
                         { filter }
                    </div>);
        previous_levels.push(level.title);
      }

    }, this);

    return (
      <div>
        { levels }
      </div>
    );

    function filteredProducts(){

      if(previous_levels.length){

        return this.props.products.filter(function(product){
          return previous_levels.some(function(level){
            return product.filters[level] === this.props.selected[level];
          }, this);
        }, this);

      } else {
        return this.props.products;
      }
    }
  }
});

ProductGroupedApp.FilterComparison = React.createClass({
  render: function(){

    if(this.props.fields.length){

      var toggleDropdown = function(){
        $(React.findDOMNode(this.refs.filter_comparison)).dropdown('toggle');
      }.bind(this);

      var table_rows = [];

      // add images
      table_rows.push(
          <tr key="headings">
            <td></td>
            {
              this.props.options.map(function(option, i){
                var image;
                if(option.image){
                  image = <img src={ option.image } className="img-responsive" />;
                }
                return <td key={ i }>{ image }</td>;
              })
            }
          </tr>
      );

      this.props.fields.forEach(function(field, i){
        table_rows.push(
          <tr key={ i }>
            <td>{ field }</td>
            {
              this.props.options.map(function(option, i){
                return <td key={ i }>{ option.comparison_data[field] }</td>;
              })
            }
          </tr>
        );
      }, this);

      return(
        <span className="dropdown" >
          <i className="fa fa-info-circle" data-toggle="dropdown" ref="filter_comparison" onMouseOver={ toggleDropdown } onMouseOut={ toggleDropdown } />
          <div className="dropdown-menu comparisons">
            <table className="table table-striped">
              { table_rows }
            </table>
          </div>
        </span>
      );

    } else {
      return <span />;
    }

  }
});

// content container
ProductGroupedApp.ProductImageThumbnail = React.createClass({
  handleClicked: function(e){
    e.preventDefault();
    this.props.onClicked(this.props.index);
  },
  render: function(){
    return(
      <a href={ this.props.image }
         className={ classNames({'thumbnail':true, 'selected':this.props.selected}) }
         onClick={ this.handleClicked }>
        <img src={ this.props.thumb } />
      </a>
    );
  }
});

ProductGroupedApp.ProductImages = React.createClass({
  getInitialState: function(){
    return {
      index: 0
    };
  },
  handleClicked: function(index){
    this.setState({
      index: index
    });
  },
  render: function(){

    var product_images = (this.props.product && this.props.product.images) || [];
    if(product_images.length){

      // resource could be an image or something in an iFrame
      var resource;
      // use this to find the resource's file extension
      var parser  = document.createElement('a');
      parser.href = product_images[this.state.index].full.toLowerCase();
      // is the resource an image
      if(parser.href.replace(parser.search, '').match(/.jp(e)?g$|.gif$|.png$/)){
        resource = <img src={ product_images[this.state.index].full } className="img-responsive center-block product-image" />;
      } else {
        resource = (
          <div className="flex-video">
              <iframe src={ product_images[this.state.index].full }
                      scrolling="no"
                      frameBorder="no"
                      className="img-responsive center-block"
                      allowFullScreen />
          </div>
        );
      }

      var thumbnails = product_images.map(function(product_image, index){
          return <ProductGroupedApp.ProductImageThumbnail image={ product_image.full }
                                                          thumb={ product_image.thumb }
                                                          index={ index }
                                                          selected={ index === this.state.index }
                                                          onClicked={ this.handleClicked } />;
      }, this);

      // key is used to reset the state back to zero when the selected product changes
      if(thumbnails.length > 1){

        return (
          <div>
            <div className="row">
              <div className="col-md-2 hidden-xs hidden-sm">
                <ul className="list-unstyled thumbnails">
                  {
                    thumbnails.map(function(thumbnail, i){
                      return <li key={i}>{ thumbnail }</li>;
                    })
                  }
                </ul>
              </div>
              <div className="col-md-10">
                { resource }
                <ProductGroupedApp.Themes themes={ this.props.themes }
                                          selected={ this.props.selected_theme }
                                          onSelect={ this.props.onSelectTheme } />
              </div>
            </div>
            <div className="row hidden-md hidden-lg thumbnails">
              {
                thumbnails.map(function(thumbnail, index){
                  return <div className="col-xs-2" key={index}>{ thumbnail }</div>;
                })
              }
            </div>
          </div>
        );
      } else {
        return (
          <div>
            { resource }
            <ProductGroupedApp.Themes themes={ this.props.themes }
                                      selected={ this.props.selected_theme }
                                      onSelect={ this.props.onSelectTheme } />
          </div>
        );
      }
    } else {

      // show a global default image
      return <ProductGroupedApp.Themes themes={ this.props.themes }
                                       selected={ this.props.selected_theme }
                                       onSelect={ this.props.onSelectTheme } />;
    }
  }
});

ProductGroupedApp.PickupLocation = React.createClass({
  render: function(){
    var location = this.props.location

    return (
      <li>
        <strong>{ location.address.city }</strong> - { location.address.street }
      </li>
    )
  }
})

ProductGroupedApp.PickupLocations = React.createClass({
  getInitialState: function() {
    return {
      postalCode: this.props.postal_code,
      locations: [],
      showZipCodeForm: (this.props.postal_code.length == 0),
      errorMessage: ""
    };
  },
  findLocations: function () {
    $.getJSON("/store-locator/nearest-pick-up-locations.json", { postal_code: this.state.postalCode, service_filter: 'SERVICE_PHOTO_CENTER_ONE_HOUR' })
      .done(function(locations) {
        if (this.isMounted()) {
          if(locations.length > 0 ) {
            this.setState({
              locations: locations,
              showZipCodeForm: false,
              errorMessage: ''
            });
          } else {
            this.setState({
              locations: locations,
              showZipCodeForm: true,
              errorMessage: 'No store found in a radius of 50 miles from your zip code. Please try again with a different zip code.'
            });
          }
        }
      }.bind(this))
      .fail(function(error) {
        if (this.isMounted()) {
          this.setState({
            postalCode: '',
            locations: [],
            showZipCodeForm: true,
            errorMessage: JSON.parse(error.responseText).error
          });
        }
      }.bind(this));
  },
  componentDidMount: function() {
    if(this.state.postalCode.length > 0) {
      this.findLocations();
    }
  },
  handleChangeClick: function(element) {
    element.preventDefault();
    $(this.getDOMNode()).toggleClass('open');
    this.setState({
      showZipCodeForm: true
    })
  },
  handleSubmit: function (postalCode) {
    this.setState({
      postalCode: postalCode
    }, function() {
      this.findLocations();
    });
  },
  render: function(){
    var errorMessageClass = "hide-header";
    if (this.state.errorMessage.length > 0) {
      errorMessageClass = "form-message"
    }

    return (
      <div className="dropdown delivery-locations">
        <a href="#" data-toggle="dropdown">
          View pickup locations <span className="caret"></span>
        </a>
        <div className="dropdown-menu">
          <ProductGroupedApp.DeliveryZipCodeHeader  showZipCodeForm= { this.state.showZipCodeForm }
                                                    postalCode= { this.state.postalCode }
                                                    handleChangeClick= { this.handleChangeClick }
                                                    handleSubmit= { this.handleSubmit }/>
          <p className={ errorMessageClass }>{ this.state.errorMessage }</p>
          <ul className="locations">
          {
            this.state.locations.map(function(location) {
              return <ProductGroupedApp.PickupLocation location={ location }/>;
            })
          }
          </ul>
        </div>
      </div>
    );
  }
});

ProductGroupedApp.DeliveryZipCodeHeader = React.createClass({
  handleSubmit: function(e) {
    e.preventDefault();

    this.props.handleSubmit(this.refs.postCode.getDOMNode().value)
  },
  render: function(){
    var dropdown_header_class = '';
    var zip_code_form_class = 'form-group';

    if(this.props.showZipCodeForm){
      dropdown_header_class = 'hide-header';
    } else {
      zip_code_form_class += ' hide-header';
    };

    return (
      <div>
        <h5 className={ dropdown_header_class }>
          Showing options for { this.props.postalCode } <a href='#' onClick={ this.props.handleChangeClick }>Change</a>
        </h5>
        <div className={ zip_code_form_class } >
          <form onSubmit={ this.handleSubmit }>
            <div> Showing options for </div>
            <input ref='postCode' className="form-control" type="text" name="postal_code" id="postal-code" defaultValue={this.props.postalCode}/>
            <input type='submit' className='btn btn-link' value='Find' />
          </form>
        </div>
      </div>
    )
  }
});

ProductGroupedApp.EstimatedArrivalDates = React.createClass({
  getInitialState: function() {
    return {
      postalCode: this.props.postal_code,
      locations: [],
      showZipCodeForm: (this.props.postal_code.length == 0),
      errorMessage: "Please insert a zip code above for show the nearest stores addresses."
    };
  },
  findLocations: function () {
    $.getJSON("/store-locator/nearest-pick-up-locations.json", { postal_code: this.state.postalCode })
      .done(function(locations) {
        if (this.isMounted()) {
          if(locations.length > 0 ) {
            this.setState({
              locations: locations,
              showZipCodeForm: false,
              errorMessage: ''
            });
          } else {
            this.setState({
              locations: locations,
              showZipCodeForm: true,
              errorMessage: 'No store found in a radius of 50 miles from your zip code. Please try again with a different zip code.'
            });
          }
        }
      }.bind(this))
      .fail(function(error) {
        if (this.isMounted()) {
          this.setState({
            postalCode: '',
            locations: [],
            showZipCodeForm: true,
            errorMessage: JSON.parse(error.responseText).error
          });
        }
      }.bind(this));
  },
  componentDidMount: function() {
    if(this.state.postalCode.length > 0) {
      this.findLocations();
    }
  },
  handleChangeClick: function(element) {
    element.preventDefault();
    $(this.getDOMNode()).toggleClass('open');
    this.setState({
      showZipCodeForm: true
    })
  },
  handleSubmit: function (postalCode) {
    this.setState({
      postalCode: postalCode
    }, function() {
      this.findLocations();
    });
  },
  render: function(){
    var errorMessageClass = "hide-header";
    if (this.state.errorMessage.length > 0) {
      errorMessageClass = "form-message"
    }
    // debugger;

    if(!this.props.deliveryDays){
      return <div/>;
    }

    return (
      <div className="dropdown delivery-locations">
        <p><strong>Delivery available to your home or store.</strong></p>
        <a href="#"
           data-toggle="dropdown">
             Estimated arrival dates <span className="caret"></span>
        </a>
        <div className="dropdown-menu pickup">
          <ProductGroupedApp.DeliveryZipCodeHeader  showZipCodeForm= { this.state.showZipCodeForm }
                                                    postalCode= { this.state.postalCode }
                                                    handleChangeClick= { this.handleChangeClick }
                                                    handleSubmit= { this.handleSubmit }/>
          <div className='delivery-options-section'>
            <div className='header-bar'>
              <div className='ship-to-home-ico'/><strong>Ship to home</strong>
            </div>
            <div className="post-delivery-dates">
              {
                this.props.deliveryDays['POST'].map(function(deliveryDay) {
                  return  <div className='post-delivery-date'>
                            <div className='delivery-type'>Standard delivery</div>
                            <div className='delivery-date'>{ deliveryDay }</div>
                          </div>
                })
              }
            </div>
          </div>
          <div className='delivery-options-section'>
            <div className='header-bar'>
              <div className='ship-to-store-ico'/><strong>Ship to store - FREE  </strong>  Available by { this.props.deliveryDays['PICKUP'][0] }

            </div>
            <p className={ errorMessageClass }>{ this.state.errorMessage }</p>
            <ul className="locations">
            {
              this.state.locations.map(function(location) {
                return <ProductGroupedApp.PickupLocation location={ location }/>;
              })
            }
            </ul>
          </div>
        </div>
      </div>
    );
  }
});

ProductGroupedApp.ProductCreate = React.createClass({
  render: function(){

    if(this.props.product){
      var price_currency_parts = ProductGroupedApp.currencyParts(this.props.product.price);

      return (
        <div>
          <h5 className="price"><span className="sup">$</span>{ price_currency_parts[0] }<span className="sup">{ price_currency_parts[1] }</span></h5>
          <a href={ this.props.product.action + '?theme=' + this.props.theme } className="btn btn-primary btn-lg btn-block" >Create Now</a>
        </div>
      );

    } else {
      return <div/>;
    }

  }
});


ProductGroupedApp.Theme = React.createClass({
  handleClicked: function(e){
    e.preventDefault();
    this.props.onClick(this.props.name);
  },
  render: function(){
    var class_names = ['thumbnail'];
    if(this.props.selected){
      class_names.push('selected');
    }

    return (
        <div className='theme col-sm-2' onClick={ this.handleClicked }>
          <a className={ classNames(class_names) } href='#' title={ this.props.title }>
            <div style={ { backgroundImage: 'url(' + this.props.thumb + ')' } } />
          </a>
        </div>
      );
  }
});

ProductGroupedApp.Themes = React.createClass({
  render: function(){

    if(this.props.themes.length > 0){

      var themes = this.props.themes.map(function(theme, i){
        return <ProductGroupedApp.Theme {...theme}
                                        selected={ this.props.selected === theme.name }
                                        key={ i }
                                        onClick={ this.props.onSelect } />
      }, this);

      var selected_theme_title = this.props.themes.reduce(function(memo, theme){
        if(this.props.selected === theme.name)
          memo = theme.title;

        return memo;
      }.bind(this), this.props.themes[0].title);

      return (
        <div className="themes">
          <h4>{ selected_theme_title }</h4>
          <div className="row thumbnails">
            { themes }
          </div>
        </div>
      );

    } else {
      return <div/>;
    }

  }
});

// main section
ProductGroupedApp.Controller = React.createClass({
  getInitialState: function(){
    return {
      data: null,
      selected_filters: {},
      theme: this.props.default_theme
    };
  },
  componentDidMount: function(){
    this.loadData();
    window.onpopstate = this.onPopState;
  },
  render: function(){

    if(!this.state.data){
      return <div />;
    }

    var products_for_theme = (function(){

      var selected_theme_name = this.state.theme && this.state.theme.name;
      var products = this.productsForCurrentDeliveryOption();
      if(selected_theme_name){
        products = products.filter(function(product){
          return product.themes.map(function(theme){return theme.name;}).indexOf(selected_theme_name) > -1;
        }, this);
      }

      return products;

    }).call(this);

    var current_product = (function(){

      var products = products_for_theme;
      for(var filter in this.state.selected_filters){
        products = products.filter(function(product){
          return product.filters[filter] === this.state.selected_filters[filter];
        }, this);
      }

      return products[0];

    }).call(this);

    setPageAttributesForCurrentProduct.call(this);
    var fulfillment_details = (function(){
      switch(this.state.selected_delivery_option_tab.id){
        case 2:
          return <ProductGroupedApp.EstimatedArrivalDates deliveryDays={ this.state.selected_delivery_option_tab.deliveryDays } postal_code={ window.postal_code } locations_url={ window.locations_url }/>
        default:
          return <ProductGroupedApp.PickupLocations postal_code={ window.postal_code } locations_url={ window.locations_url }/>
      }

    }).call(this);

    var selected_delivery_option_ids = (function(){

      return this.state.delivery_option_tabs.reduce(function(memo, delivery_option_tab){
        if(delivery_option_tab.id === this.state.selected_delivery_option_tab.id){
          memo = delivery_option_tab.keys;
        }
        return memo;
      }.bind(this), null);

    }).call(this);

    return (
        <div className={ classNames({'has-delivery-options':this.state.delivery_option_tabs.length > 1}) }>
          <h1>{this.state.data.title}</h1>
          <div className="row">
            <div className="col-md-6 product-images">
              <ProductGroupedApp.ProductImages product={ current_product }
                                               key={ current_product && current_product.action }
                                               themes={ current_product ? current_product.themes : [] }
                                               selected_theme={ this.state.theme }
                                               onSelectTheme={ this.handleThemeSelect } />
            </div>
            <div className="col-md-6">
              <ProductGroupedApp.DeliveryOptionTabs
                  options       = { this.state.delivery_option_tabs }
                  selectedId    = { this.state.selected_delivery_option_tab.id }
                  onTabSelected = { this.handleDeliveryOptionTabSelected } />
              <div className="filters-and-product box-shadow">
                <div className="filters">
                  <ProductGroupedApp.Filters
                      levels                    = { this.state.data.filters }
                      selected                  = { this.state.selected_filters }
                      products                  = { products_for_theme }
                      selected_delivery_options = { selected_delivery_option_ids }
                      onCriteronSelected        = { this.handleFilterCriteronSelected } />
                </div>
                <div className="product">
                  <div className="row">
                    <div className="col-sm-8">
                      { fulfillment_details }
                    </div>
                    <div className="col-sm-4">
                      <ProductGroupedApp.ProductCreate product={ current_product } theme={ this.state.theme } />
                    </div>
                  </div>
                </div>
              </div>
            </div>
          </div>
          <hr/>
          <div dangerouslySetInnerHTML={{__html: (current_product && current_product.description) || this.state.data.description}} />
        </div>
      );


    function setPageAttributesForCurrentProduct(){

      if(!current_product){
        return;
      }

      var theme_name = (current_product.themes || []).reduce(function(memo, theme){
            if(theme.title === this.state.theme){
              memo = theme.name;
            }
            return memo;
          }.bind(this), '');

      document.title = current_product.title + ' - ' + theme_name + ' | ' + this.props.default_page_title;

      if(window['history']){

        var hash = window.location.hash.split('#')[1];
        if(!hash){
          hash = '|';
        }

        var parts = hash.split('|');
        var current_hash_product_id = (parts[0].match(/^(\d+)-/) || [])[1];
        var current_hash_theme = parts[1];

        if(current_hash_product_id != current_product.id || current_hash_theme != this.state.theme){
          // url doesn't match product
          var pushStateMethod = window.history.replaceState;
          if(window.history.state){
            pushStateMethod = window.history.pushState;
          }

          var action_parts = current_product.action.split('/');
          var path = window.location.origin + window.location.pathname + '#' + action_parts[action_parts.length-1];
          if(this.state.theme){
            path += '|' + this.state.theme;
          }

          pushStateMethod.call(window.history, {
              product_id: current_product.id,
              theme: this.state.theme
            },
            '',
            path
          );
        }
      }
    }
  },
  handleDeliveryOptionTabSelected: function(delivery_option_tab){

    this.setState({
      selected_delivery_option_tab: delivery_option_tab
    }, function(){
      this.setDefaultFilterSelection();
    });

  },
  handleFilterCriteronSelected: function(title, criteron){

    var selected_filters = this.state.selected_filters;
    selected_filters[title] = criteron;

    // need to reset any filters which follow on from this one.
    var passed_current = false;
    this.state.data.filters.map(function(filter){
      return filter.title;
    }).forEach(function(filter_title){
      if(passed_current){
        delete selected_filters[filter_title];
      }
      if(filter_title === title){
        passed_current = true;
      }
    });

    // find product matching filter presently
    var default_product = this.productsForCurrentDeliveryOption().filter(function(product){
      var is_a_match = false;
      for(var key in selected_filters){
        is_a_match = product.filters[key] === selected_filters[key];
      }
      return is_a_match;
    }, this)[0];

    // set any empty filters with that for first matching product
    if(default_product){
      this.state.data.filters.map(function(filter){
        return filter.title;
      }).forEach(function(filter_title){
        if(!selected_filters[filter_title]){
          selected_filters[filter_title] = default_product.filters[filter_title];
        }
      });
    }

    this.setState({
      selected_filters: selected_filters
    });

  },
  productsForCurrentDeliveryOption: function(selected_delivery_option_tab_id){

    selected_delivery_option_tab_id || (selected_delivery_option_tab_id = this.state.selected_delivery_option_tab.id);
    var delivery_option_keys = this.state.delivery_option_tabs.reduce(function(memo, delivery_option_tab){
      if(delivery_option_tab.id === selected_delivery_option_tab_id){
        memo = memo.concat(delivery_option_tab.keys);
      }
      return memo;
    }, []);

    return this.state.data.products.filter(function(product){
      return product.delivery_options && product.delivery_options.some(function(delivery_option_key){
        return delivery_option_keys.indexOf(delivery_option_key) > -1;
      });
    });

  },
  setDefaultFilterSelection: function(default_product){

    var product = default_product;

    if(typeof default_product === 'number'){
      if(default_product){
        product = this.state.data.products.filter(function(product){
          return product.id === default_product;
        })[0];
      }
    }
    product || (product = this.productsForCurrentDeliveryOption()[0]);

    this.setState({
      selected_filters: this.state.data.filters.map(function(filter){
                                                      return filter.title;
                                                    }).
                                                    reduce(function(memo, filter_title){
                                                      memo[filter_title] = product && product.filters[filter_title];
                                                      return memo;
                                                    }.bind(this), {})

    });

  },
  handleThemeSelect: function(theme){
    this.setState({
      theme: theme
    });
  },
  loadData: function(){

    var that = this;
    var xhr = new XMLHttpRequest();
    xhr.onreadystatechange = function(){
      if(this.readyState === XMLHttpRequest.DONE){
        if(this.status === 200){

          var data = JSON.parse(this.responseText);
          var delivery_option_tabs = calculateDeliveryOptionTabs(data);

          that.setState({
            data: data,
            delivery_option_tabs: delivery_option_tabs,
            selected_delivery_option_tab: delivery_option_tabs[0]
          });

          var default_product_id = that.props.default_product_id || (data.products[0] && data.products[0].id);
          if(default_product_id){

            // find product with the default_product_id
            var default_product = data.products.reduce(function(memo, product){
              if(product.id === default_product_id){
                memo = product;
              }
              return memo;
            }.bind(that), null);

            if(default_product){

              var default_delivery_option_id = default_product.delivery_options[0];
              var default_delivery_option_tab = delivery_option_tabs.reduce(function(memo, delivery_option_tab){
                if(delivery_option_tab.id === default_delivery_option_id){
                  memo = delivery_option_tab;
                }
                return memo;
              }, delivery_option_tabs[0]);

              that.setState({
                selected_delivery_option_tab: default_delivery_option_tab
              });
              that.setDefaultFilterSelection(default_product);
            }
          }
        }
      }
    };
    xhr.open("GET", this.props.url);
    xhr.send();


    function calculateDeliveryOptionTabs(data){

      var pickup_tab={
        id:   1,
        keys: [],
        deliveryDays: {},
        text: <div><p className="lead">Need it fast?</p><p>Same day pickup</p></div>
      },
      ship_tab={
        id:   2,
        keys: [],
        deliveryDays: {},
        text: <div><p className="lead">Want more selection?</p><p>Ship to home or store</p></div>
      };

      var product_delivery_options, tab, one_hour_available;

      data.delivery_options
          .sort(function(a,b){
              return a.time_hours - b.time_hours;
          })
          .forEach(function(delivery_option){

            switch(delivery_option.type.toUpperCase()){
              case 'PICKUP':
                switch(delivery_option.timescale.toLowerCase()){
                  case '1 hour':
                    tab = pickup_tab;
                    one_hour_available = deliveryOptionKeyHasProduct(delivery_option.key);
                    break;
                  case '1 day':
                    tab = pickup_tab;
                    break;
                  case '5 days':
                    tab = ship_tab;
                    tab.deliveryDays['PICKUP'] = (tab.deliveryDays['PICKUP'] || []).concat(delivery_option.delivery_day)
                    break;
                }
                break;
              case 'POST':
                tab = ship_tab;
                tab.deliveryDays['POST'] = (tab.deliveryDays['POST'] || []).concat(delivery_option.delivery_day)
                break;
            }

            if(tab){
              tab.keys.push(delivery_option.key);
              tab.has_product = tab.has_product || deliveryOptionKeyHasProduct(delivery_option.key);
            }
          });

      if(one_hour_available){
        pickup_tab.text = <div><p className="lead">Need it fast?</p><p>Available as soon as 1 hour</p></div>;
      }

      // order tabs and remove any without products
      return (function(){
        var i, len, ref, results, tab;
        ref = [pickup_tab, ship_tab];
        results = [];
        for (i = 0, len = ref.length; i < len; i++){
          tab = ref[i];
          if (tab.has_product){
            results.push(tab);
          }
        }
        return results;
      })();

      function deliveryOptionKeyHasProduct(key){
        if(!product_delivery_options){
          product_delivery_options = [];
          data.products.forEach(function(product){
            product.delivery_options && product.delivery_options.forEach(function(delivery_option){
              if(product_delivery_options.indexOf(delivery_option) === -1){
                product_delivery_options.push(delivery_option);
              }
            });
          });
        }

        return product_delivery_options.indexOf(key) > -1;
      }
    }
  },
  onPopState: function(e){
    this.setDefaultFilterSelection(e.state.product_id);
    this.setState({
      theme: e.state.theme
    });
  }
});

React.render(
  <ProductGroupedApp.Controller url={ window.location.pathname + '.json' }
                                default_product_id={ Number((window.location.hash.match(/^#(\d+)-/) || [])[1]) }
                                default_theme={ window.location.hash.split('|')[1] }
                                default_page_title={ document.title }/>,
  document.getElementById('content')
);
